@@ -1,17 +1,20 @@
 """An interface to MorphoDiTa tokenizers.
 
 """
+from __future__ import annotations
+import collections.abc as c
+from pathlib import Path
+import typing as t
+
 import ufal.morphodita as ufal
 
 
 class Tokenizer:
     """A wrapper API around the tokenizers offered by MorphoDiTa.
 
-    :param tokenizer_type: Type of the requested tokenizer (cf. below for
-        possible values).
-    :type tokenizer_type: str
+    :param tokenizer_type: Tokenizer type, see below for possible values.
 
-    `tokenizer_type` is typically one of:
+    ``tokenizer_type`` is typically one of:
 
     - ``"czech"``: a tokenizer tuned for Czech
     - ``"english"``: a tokenizer tuned for English
@@ -26,30 +29,42 @@ class Tokenizer:
 
     """
 
-    def __init__(self, tokenizer_type):
+    def __init__(self, tokenizer_type: str):
         constructor_name = "new" + tokenizer_type.capitalize() + "Tokenizer"
-        self.tokenizer_constructor = getattr(ufal.Tokenizer, constructor_name)  # type: ignore
+        self.tokenizer_constructor = getattr(ufal.Tokenizer, constructor_name)
 
-    @staticmethod
-    def from_tagger(tagger_path):
+    @classmethod
+    def from_tagger(cls, tagger_path: str | Path) -> t.Self:
         """Load tokenizer associated with tagger file."""
-        self = Tokenizer("generic")
-        tagger = ufal.Tagger.load(str(tagger_path))  # type: ignore
+        self = cls("generic")
+        tagger = ufal.Tagger.load(str(tagger_path))
         self.tokenizer_constructor = tagger.newTokenizer
         if self.tokenizer_constructor() is None:
             raise RuntimeError(f"The tagger {tagger_path} has no associated tokenizer.")
         return self
 
-    def tokenize(self, text, sents=False):
+    @t.overload
+    def tokenize(self, text: str, sents: t.Literal[False]) -> c.Iterator[str]:
+        ...
+
+    @t.overload
+    def tokenize(self, text: str, sents: t.Literal[True]) -> c.Iterator[list[str]]:
+        ...
+
+    @t.overload
+    def tokenize(
+        self, text: str, sents: bool = ...
+    ) -> c.Iterator[str] | c.Iterator[list[str]]:
+        ...
+
+    def tokenize(
+        self, text: str, sents: bool = False
+    ) -> c.Iterator[str] | c.Iterator[list[str]]:
         """Tokenize `text`.
 
         :param text: Text to tokenize.
-        :type text: str
-        :param sents: Whether to signal sentence boundaries by outputting a
-            sequence of lists (sentences).
-        :type sents: bool
-        :return: An iterator over the tokenized text, possibly grouped into
-            sentences if ``sents=True``.
+        :param sents: If ``True``, return an iterator of lists of tokens, each
+            list being a sentence, instead of a flat iterator of tokens.
 
         Note that MorphoDiTa performs both sentence splitting and tokenization
         at the same time, but this method iterates over tokens without sentence
@@ -73,14 +88,12 @@ class Tokenizer:
         ['foo', 'bar', 'baz']
 
         """
-        # this is more elegant than just segfaulting in the MorphoDiTa C++ library if None is
-        # passed...
+        # this is more elegant than just segfaulting in the MorphoDiTa C++
+        # library if None is passed...
         if not isinstance(text, str):
-            raise TypeError(
-                "``text`` should be a str, you passed in {}.".format(type(text))
-            )
-        forms = ufal.Forms()  # type: ignore
-        token_ranges = ufal.TokenRanges()  # type: ignore
+            raise TypeError(f"``text`` should be str, you passed in {type(text)}.")
+        forms = ufal.Forms()
+        token_ranges = ufal.TokenRanges()
         tokenizer = self.tokenizer_constructor()
         tokenizer.setText(text)
         while tokenizer.nextSentence(forms, token_ranges):
